#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/usb.h>

typedef unsigned char __u8;
typedef unsigned short __u16;

typedef struct {
        __u8 requesttype;
        __u8 request;
        __u16 value;
        __u16 index;
        __u16 length;
} devrequest __attribute__ ((packed));

typedef unsigned char uchar;
#include "quickcamex.h"
#include "stv6x.h"
#define EMPTY_TAG 0xaa

void ctrl_out(int fd, int reg, int len, __u8 *buffer) {
	ctrl_transfer_t ct;

	ct.setup.requesttype=0xc0;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=len;

	memcpy(ct.data, buffer, len);

	ioctl(fd,IOCTL_QUICKCAM_CTRLR,&ct);
	return;
}

/*-----------------------------------------------------------------*/
// Read STV-register
int quick_reg_in(int fd, int reg)
{
	ctrl_transfer_t ct;

	ct.setup.requesttype=0xc0;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=1;

	ct.data[0] = EMPTY_TAG;

	ioctl(fd,IOCTL_QUICKCAM_CTRLR,&ct);
	return ct.data[0];
}
/*-----------------------------------------------------------------*/
// Read STV-register
int quick_reg_out(int fd, int reg, int value)
{
	ctrl_transfer_t ct;

	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=1;

	ct.data[0] = value;

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
	return ct.data[0];
}

void quick_i2c_out_vv(int fd, int reg, int val)
{
	ctrl_transfer_t ct;
	
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE;
	ct.setup.index=0;
	ct.size=0x23;

	ct.data[0x20]=0x20;
	ct.data[0x21]=0; // 1 value
	ct.data[0x22]=1; // Write cmd

	ct.data[0]=reg;
	ct.data[0x10]=val;

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);

	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=0x1704;
	ct.setup.index=0;
	ct.size=0x1;
	ct.data[0]=1;
	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
// I2C-Output one byte for HDCS1000
void quick_i2c_out(int fd, int reg, int val)
{
	ctrl_transfer_t ct;
	
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE;
	ct.setup.index=0;
	ct.size=0x23;

	ct.data[0x20]=0x20;
	ct.data[0x21]=0; // 1 value
	ct.data[0x22]=1; // Write cmd

	ct.data[0]=reg;
	ct.data[0x10]=val;

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);	
}

/*-----------------------------------------------------------------*/
int main(int argc, char** argv)
{
	int fd;
	char *driver = "/dev/quick";
	if (argc >= 2)
	  driver = argv[1];

	// Open the driver...
	fd=open(driver,O_RDWR);
	if (fd==-1)
	{
		perror("/dev/quick major 180, minor 80");
		exit(0);
	}

	quick_reg_out(fd, 0x1440, 0);
	quick_i2c_out_vv(fd, 0x10, 0x4);
	quick_i2c_out_vv(fd, 0x10, 0x1);
	quick_reg_out(fd, 0x1620, 0x80);
	quick_reg_out(fd, 0x1620, 0x00);

	return 0;
}


