/*****************************************************************************
 *
 *  quickcam.c - Logitech Quickcam Express Viewer 
 *               (with HDCS1000/PB0100 image sensor)
 *
 *  Copyright (C) 2000 Georg Acher (acher@in.tum.de)
 *  Copyright (C) 2000 Mark Cave-Ayland (mca198@ecs.soton.ac.uk)
 *
 *  Bugs:  Auto-Exposure needs a bit more intelligence
 *
 *   Needs quickcamex-module and USB support!
 *
 *****************************************************************************/
/* Cam variations of Logitech Quickcam Express:
   P/N 861037:      Sensor HDCS1000        ASIC STV0600
   P/N 861050-0010: Sensor HDCS1000        ASIC STV0600
   P/N 861050-0020: Sensor Photobit PB100  ASIC STV0600
   P/N 861055:      Sensor ST VV6410       ASIC STV0610 ("LEGO cam")
*/

/* Streamformat, as delivered by the camera with STV0600:

   Image starts with 80 02 00 00 80 01 00 00
   Chunk-Header  02 00 HH LL, followed by chunk with HHLL bytes
   Raw image data for Bayer-RGB-matrix:
   G R for even rows
   B G for odd rows
*/
#define DECODING_HACK
//#define FILE_MODE

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/extensions/XShm.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#ifdef DECODING_HACK
#include <sys/mman.h>
#endif DECODING_HACK

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/usb.h>

#define max(a,b) ((a)>(b)?(a):(b))
#define min(a,b) ((a)<(b)?(a):(b))

typedef unsigned char  __u8;
typedef unsigned short __u16;
typedef unsigned long  __u32;

typedef struct {
        __u8 requesttype;
        __u8 request;
        __u16 value;
        __u16 index;
        __u16 length;
} devrequest __attribute__ ((packed));

typedef unsigned char uchar;

#include "quickcamex.h"
#include "stv6x.h"
#include "hdcs1000.h"
#include "pb0100.h"
#include "vv6410.h"


// Sensor types...
#define SENSOR_HDCS 0x0
#define SENSOR_PB 0x1
#define SENSOR_VV 0x2

// X-STUFF
#define XW 360
#define XH 292

GC      gc;
GC      gc1,gcn;
Window  win;
Font font;
Display *dpy=0;
int     screen, scrnWidth = XW, scrnHeight = XH;
Pixmap  buffer;
int scrnW2,scrnH2;
unsigned short flags = 0;
char *text;
XColor colors[16];
XImage* img;
char *s;
XShmSegmentInfo shminfo;
int depth;
int bypp;

// Other stuff

unsigned char SENSOR_ADDR;
char buf[32768];
int po;
int shutter_val=0x80;
int gain=0xc0;
int white_balance=0;

void init_vv6410(int fd, int mode);
void init_hdcs1000(int fd, int mode);

/*-----------------------------------------------------------------*/
void initializeX (int width, int height)
{
	XGCValues *xgc;

	if (dpy!=0) 
		return;

	xgc=( XGCValues *) malloc(sizeof(XGCValues) );

	if (!(dpy = XOpenDisplay (""))) 
	{
		(void) fprintf (stderr, "Error:  Can't open display\n");
		exit (1);
	}

	screen = DefaultScreen (dpy);   

	win = XCreateSimpleWindow (dpy, DefaultRootWindow (dpy),
				   0, 0, width, height, 1,
				   WhitePixel (dpy, screen),
				   BlackPixel (dpy, screen));

	XSelectInput (dpy, win, 
		      ExposureMask | StructureNotifyMask
		      | KeyPressMask | ButtonPressMask
		      |VisibilityChangeMask |PropertyChangeMask);

	depth=DefaultDepth(dpy,screen);
	if ((depth!=24)&&(depth!=16))
	{
		fprintf(stderr,"quickcam works only with 16/24Bit displays\n");
		exit(0);
	}
	
	if (depth==24)
		bypp=4;
	else
		bypp=2;

	XStoreName (dpy, win, "Quickcam Express");
	XMapWindow (dpy, win);

	gc = XCreateGC (dpy, win, 0,  xgc);

	for (;;) 
	{		
		XEvent event;
		XNextEvent(dpy, &event);
		if (event.type == Expose) break;
	}
}
/*-----------------------------------------------------------------*/
void NewPictImage(int width, int height)
{       
        img=NULL;
  
	img = XShmCreateImage(dpy, None, depth, ZPixmap, NULL,
			      &shminfo, width, height+16);

	if (img == NULL) 
	{
		fprintf(stderr, "Shared memory error.\n");
		fprintf(stderr, "Ximage error.\n");
		exit(0);
	}

	/* Success here, continue. */

	shminfo.shmid = shmget(IPC_PRIVATE, ((XW)*bypp*(XH+16)), 
			       IPC_CREAT|0777);
    
	if (shminfo.shmid < 0) 
	{
		XDestroyImage(img);
		img = NULL;
		fprintf(stderr, "Shared memory error.\n");
		fprintf(stderr, "Seg. id. error.\n");
		exit(0);
	}

	shminfo.shmaddr = (char *) shmat(shminfo.shmid, 0, 0);
	if (shminfo.shmaddr == ((char *) -1)) 
	{
		XDestroyImage(img);
		img = NULL;
		fprintf(stderr, "Shared memory error.\n");
		fprintf(stderr, "Address error.\n");
		exit(0);
	}

	img->data = shminfo.shmaddr;
	shminfo.readOnly = False;

	XShmAttach(dpy, &(shminfo));
	XSync(dpy, False);

	fprintf(stderr, "Sharing memory.\n");
}

/*-----------------------------------------------------------------*/
void ctrl_out(int fd, int reg, int len, char *data)
{
	quick_reg_outn(fd, reg, len, data);
}

/*-----------------------------------------------------------------*/
void init_855(int dev)
{

	fprintf(stderr,"INIT LEGO CAM\n");
	#include "mylego.init"
//	#include "lego0.init"
//	#include "lego1.init"
//	init_vv6410(dev,0);
//	exit(0);
}
/*-----------------------------------------------------------------*/
// Find out which sensor the camera has...
int quick_detect_sensor(int fd)
{
	ctrl_transfer_t ct;
	int cam_id;

	// Identify
	cam_id=ioctl(fd,IOCTL_QUICKCAM_ID, 0);

	fprintf(stderr,"Camera ID %04x\n",cam_id);

	if (cam_id==0x850)
	{
		return SENSOR_VV;
	}
	
// HDSC* detection
	// Set quickcam i2c for 8-bit regs?
	quick_reg_out1(fd, STV_REG23, 0);
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE1;
	ct.setup.index=0;
	ct.size=0x23;
	
	ct.data[0]= HDCS_IDENT + 1;

	ct.data[0x20]= HDCS_ADDR;
	ct.data[0x21]=0; // 1 value
	ct.data[0x22]=3; // Write cmd

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);

	ct.setup.requesttype=0xc0;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_READ;
	ct.setup.index=0;
	ct.size=0x1;

	ct.data[0]=0xaa;

	 ioctl(fd,IOCTL_QUICKCAM_CTRLR,&ct);

	printf("HDCS1000-sensor ident: %x\n",ct.data[0]);
	if (ct.data[0]==8)
	{
		fprintf(stderr, "This Quickcam has a HDCS1000 sensor!\n");
		SENSOR_ADDR = HDCS_ADDR;
		return SENSOR_HDCS;
	}	

// Photobit detection
	// Set quickcam i2c for 16-bit regs?
	quick_reg_out1(fd, STV_REG23, 1);

	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE1;
	ct.setup.index=0;
	ct.size=0x23;
	
	ct.data[0]= PB_IDENT;

	ct.data[0x20]= PB_ADDR;
	ct.data[0x21]=0; // 1 value
	ct.data[0x22]=3; // Write cmd

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);

	ct.setup.requesttype=0xc0;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_READ;
	ct.setup.index=0;
	ct.size=0x2;

	ct.data[0]=ct.data[1]=0xaa;

	ioctl(fd,IOCTL_QUICKCAM_CTRLR,&ct);
	
	printf("Photobit-sensor ident: %x%x\n",ct.data[0],ct.data[1]);

	if (ct.data[1]==0x64)
	{
		fprintf(stderr, "This Quickcam has a Photobit PB100 sensor\n");
		SENSOR_ADDR = PB_ADDR;
		return SENSOR_PB;
	}
// Set quickcam i2c for 8-bit regs?
	quick_reg_out1(fd, STV_REG23, 0);
	fprintf(stderr, "This Quickcam has an unknown and unsupported sensor!\n");
	exit(0);
}

/*-----------------------------------------------------------------*/
unsigned int getn(int fd)
{
	uchar c;
	int l;

	c=buf[po];
	po++;

	if (po>=16384)
	{
		l=read(fd,buf,16384);
#ifdef FILE_MODE
		if (l!=16384)
			exit(0);
#endif
		po=0;
	}
	return c;
}

void getbuf(int fd,__u8 *buffer, int count)
{
	int l;
	int len;
 retry:
	len = count;
	if (len >= 16384-po)
		len = 16384 - po;
	memcpy(buffer, buf+po, len);
	count -= len;
	buffer += len;
	po += len;

	if (po>=16384)
	{
		l=read(fd,buf,16384);
#ifdef FILE_MODE
		if (l!=16384)
			exit(0);
#endif
		po=0;
		if (count > 0)
			goto retry;
	}
}
/*-----------------------------------------------------------------*/
void set_shutter(int fd, int sval, int xval)
{
	ctrl_transfer_t ct;
	int n;

	printf("SHUTTER %i\n",sval);
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE;
	ct.setup.index=0;
	ct.size=0x23;

	ct.data[0x20]=SENSOR_ADDR;
	ct.data[0x21]=5; // 6 values
	ct.data[0x22]=1; // Write cmd

	n=0;
	ct.data[n]=HDCS_CONTROL;
	ct.data[0x10+n++]=0;
	ct.data[n]=HDCS_ROWEXPL;
	ct.data[0x10+n++]=sval&255;
	ct.data[n]=HDCS_ROWEXPH;
	ct.data[0x10+n++]=sval>>8;
	ct.data[n]=HDCS_SROWEXPL;
	ct.data[0x10+n++]=xval&255;
	ct.data[n]=HDCS_SROWEXPH;
	ct.data[0x10+n++]=xval>>8;
	ct.data[n]=HDCS_CONTROL;
	ct.data[0x10+n++]=4;

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
void set_exposure(int fd, int val)
{
	int og=gain;

	printf("%i \n",val);
	if (val>200)
		gain-=4+(val-200)/4;
	if (val<170)
		gain+=4+(170-val)/4;

	if (gain<=40)
	{
		if (shutter_val>16)
		{
			shutter_val-=16;
			gain+=16;
			gain=min(255,gain);
			set_shutter(fd,shutter_val,0x100);
		}
		else
			if (gain<=2)
				gain=2;
	}

	gain=min(gain,400);

	if ((gain+abs(white_balance))>255 && (shutter_val<500))
	{		
		shutter_val+=16;	
		gain-=16;
		gain=max(0,gain);
		set_shutter(fd,shutter_val,0x100);
	}
	if (og!=gain)
	{
		ctrl_transfer_t ct;
		int gx;
		gx=gain/2;
		printf("GAIN %i\n", gain);

		ct.setup.requesttype=0x40;
		ct.setup.request=0x04;
		ct.setup.value=0x400;
		ct.setup.index=0;
		ct.size=0x23;

		ct.data[0x20]=HDCS_ADDR;
		ct.data[0x21]=3; // 4 values
		ct.data[0x22]=1; // Write cmd

		ct.data[0]=HDCS_GAIN_GREEN1;
		ct.data[0x10]=128+max(0,min(127,gx));
		ct.data[1]=HDCS_GAIN_RED;
		ct.data[0x11]=128+max(0,min(127,gx+white_balance));
		ct.data[2]=HDCS_GAIN_BLUE;
		ct.data[0x12]=128+max(0,min(127,gx-white_balance));
		ct.data[3]=HDCS_GAIN_GREEN2;
		ct.data[0x13]=128+max(0,min(127,gx));

		ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);

	}
}/*-----------------------------------------------------------------*/
void init_hdcs1000(int fd, int mode)
{
	fprintf(stderr,"Init HDCS1000 sensor\n");

	quick_reg_out1(fd, STV_REG23, 1);
	quick_reg_out1(fd, STV_REG23, 0);


	quick_i2c_out(fd, HDCS_CONTROL, 1); // reset
	quick_i2c_out(fd, HDCS_CONTROL, 0);

	quick_i2c_out(fd, HDCS_IMASK, 0x00);

	quick_i2c_out(fd, HDCS_GAIN_GREEN1, gain);
	quick_i2c_out(fd, HDCS_GAIN_RED,    max(0,min(255,gain+white_balance)));
	quick_i2c_out(fd, HDCS_GAIN_BLUE,   max(0,min(255,gain-white_balance)));
	quick_i2c_out(fd, HDCS_GAIN_GREEN2, gain);

	quick_reg_out1(fd, STV_REG00, 0x1d);
	quick_reg_out1(fd, STV_REG04, 0x07);
	quick_reg_out1(fd, STV_REG03, 0x95);
	
	quick_reg_out1(fd, STV_REG23, 0);

	quick_reg_out1(fd, STV_SCAN_RATE, (mode?0x10:0x20)); // larger -> slower

	quick_reg_out2(fd, STV_ISO_SIZE, 0x3ff); // ISO-Size 

	quick_reg_out1(fd, STV_Y_CTRL, (mode?2:1)); // 1: Y-full, 2: y-half
	quick_reg_out1(fd, STV_X_CTRL, (mode?6:0x0a)); // 06/0a : Half/Full

	quick_i2c_out(fd, HDCS_FWROW, 2);
	quick_i2c_out(fd, HDCS_FWCOL, 4);
	quick_i2c_out(fd, HDCS_LWROW, 0x4c);
	quick_i2c_out(fd, HDCS_LWCOL, 0x5b);
	
	// if you get an unsynchronized picture very often, increase the 6...
	quick_i2c_out(fd, HDCS_TCTRL, 0x6); // 0x7-0x50

	quick_i2c_out(fd, HDCS_SROWEXPL, 0x0); // ??
	quick_i2c_out(fd, HDCS_SROWEXPH, 0x0); //

	quick_reg_out1(fd, STV_REG01, 0xb5);  // ?
	quick_reg_out1(fd, STV_REG02, 0xa8);  // ?

	quick_i2c_out(fd, HDCS_PCTRL, 0x63);
	quick_i2c_out(fd, HDCS_PDRV, 0xff);
	quick_i2c_out(fd, HDCS_ICTRL, 0x20);
	quick_i2c_out(fd, HDCS_ITMG, 0x11);
	quick_i2c_out(fd, HDCS_CONFIG, 0x08);

	quick_i2c_out(fd, HDCS_ADCCTRL, 10); // 10 Bit ADC (and 5Bit CMOS noise...)

	set_shutter(fd,shutter_val,0x100);
	quick_i2c_out(fd, HDCS_CONTROL, 4);

}
/*-----------------------------------------------------------------*/
void init_pb100(int fd, int mode)
{
	fprintf(stderr,"Init PB100 sensor\n");

	quick_reg_out1(fd, STV_REG23, 1);
	quick_reg_out1(fd, STV_REG00, 1);
	quick_reg_out1(fd, STV_SCAN_RATE, 0);

	// Reset sensor
        quick_i2c_out2(fd, PB_RESET, 1); // reset
        quick_i2c_out2(fd, PB_RESET, 0);
	
        // Enable auto-exposure...
        quick_i2c_out2(fd, PB_EXPGAIN, 17);
        
        // Other gain stuff...
        quick_i2c_out2(fd, PB_PREADCTRL, 0x1444);
        quick_i2c_out2(fd, PB_VOFFSET, 0x14);
        quick_i2c_out2(fd, PB_ADCGAINH, 0xd);
        quick_i2c_out2(fd, PB_ADCGAINL, 0x1);
        
        // Set individual colour gains
        quick_i2c_out2(fd, PB_RGAIN, 0xc0);
        quick_i2c_out2(fd, PB_G1GAIN, 0xc0);
        quick_i2c_out2(fd, PB_G2GAIN, 0xc0);
        quick_i2c_out2(fd, PB_BGAIN, 0xc0);
	// ???
        quick_reg_out1(fd, STV_REG04, 0x07);
        quick_reg_out1(fd, STV_REG03, 0x45);
        quick_reg_out1(fd, STV_REG00, 0x11);

        // Set screen output size
        quick_reg_out1(fd, STV_Y_CTRL, (mode?2:1)); // 1: Y-full, 2: y-half
        quick_reg_out1(fd, STV_X_CTRL, (mode?6:0x0a)); // 06/0a : Half/Full

        quick_reg_out2(fd, STV_ISO_SIZE, 0x27b); // ISO-Size 

        // Setup sensor window
        quick_i2c_out2(fd, PB_RSTART, 0);
        quick_i2c_out2(fd, PB_CSTART, 0);
        quick_i2c_out2(fd, PB_RWSIZE, 0xf7);
        quick_i2c_out2(fd, PB_CWSIZE, 0x13f);

        // Scan rate?
        quick_reg_out1(fd, STV_SCAN_RATE, (mode?0x10:0x20)); // larger -> slower
                
        // Scan/timing for the sensor
        quick_i2c_out2(fd, PB_ROWSPEED, 0x1a);
        quick_i2c_out2(fd, PB_CFILLIN, 0x2f);
	quick_i2c_out2(fd, PB_VBL, 0);
        quick_i2c_out2(fd, PB_FINTTIME, 0);
        quick_i2c_out2(fd, PB_RINTTIME, 0x7b);

        quick_reg_out1(fd, STV_REG01, 0xc2);
        quick_reg_out1(fd, STV_REG02, 0xb0);

}
/*-----------------------------------------------------------------*/
void init_vv6410(int fd, int mode)
{
	fprintf(stderr,"Init VV6410 sensor, but don't expect any pictures!\n");
	quick_i2c_out_vv(fd, VV_LINE_LENGTHH, 0x01);
	quick_i2c_out_vv(fd, VV_LINE_LENGTHL, 0x9f);

	quick_i2c_out_vv(fd, VV_FIELD_LENGTHH, 0x01);
	quick_i2c_out_vv(fd, VV_FIELD_LENGTHL, 0x5f);
#if 1
	quick_i2c_out_vv(fd, VV_CLK_DIV, 0x01);

	quick_i2c_out_vv(fd, VV_FINEH, 0x01);
	quick_i2c_out_vv(fd, VV_FINEL, 0x80);
	quick_i2c_out_vv(fd, VV_COARSEH, 0x00);
	quick_i2c_out_vv(fd, VV_COARSEL, 0xc0);

	quick_i2c_out_vv(fd, VV_SETUP0, 0x08);
	quick_i2c_out_vv(fd, VV_SETUP1, 0x58);

	quick_i2c_out_vv(fd, VV_FG_MODES, 0x55);
	quick_i2c_out_vv(fd, VV_DATA_FORMAT, 0x81);
	quick_i2c_out_vv(fd, VV_OP_FORMAT, 0x18);
	quick_i2c_out_vv(fd, VV_AS0, 0x5e);
	quick_i2c_out_vv(fd, VV_AT1, 0x00);
	quick_i2c_out_vv(fd, VV_PIN_MAPPING, 0x00);

	quick_i2c_out_vv(fd, VV_PIN_MAPPING, 0x00);

	quick_i2c_out_vv(fd, VV_DARK_OFFSET_SETUP, 0x0);

	quick_i2c_out_vv(fd, VV_BLACK_OFFSET_SETUP, 0x0);
#endif
}
/*-----------------------------------------------------------------*/

void set_pixel24(int mode, int x, int y, int yy, int xxx, int width,
		 long long *mid_valuer, long long *mid_valueg, long long *mid_valueb)
{
	uchar *o;

	o=((char*)shminfo.shmaddr)+4*x+(width*4*(yy));
 
	if (!(y&1)) // even row
	{
		if ((x&1)) // odd column, red
		{
			*mid_valuer+=xxx;
			*(o+2)=xxx;
			*(o+2-4)=xxx;
	      
			if (!mode)
			{
				*(o+2+4*width)=xxx;
				*(o+2-4+4*width)=xxx;
			}
		}
		else  // green
		{
			*(o+1)=xxx;		     
			*(o+1+4)=xxx;
			*mid_valueg+=xxx;
		}
	}
	else
	{
		if ((x&1)) // odd column green
		{	
			*mid_valueg+=xxx;
			*(o+1)=xxx;
			*(o+1-4)=xxx;			
		}
		else //blue
		{
			*mid_valueb+=xxx;
			*(o)=xxx;
			*(o+4)=xxx;
			if (!mode)
			{
				*(o-4*width)=xxx;
				*(o+4-4*width)=xxx;	
			}
		}
	}
}

void set_rgbpixel24(int x, int y, int r, int g, int b, int width)
{
	uchar *o;

	o=((char*)shminfo.shmaddr)+4*x+(width*4*y);
	*(o+2) = r;
	*(o+1) = g;
	*(o  ) = b;
}
void set_rgbpixel16(int x, int y, int r, int g, int b, int width) 
{
	ushort *o;

	o = ((ushort*)shminfo.shmaddr)+x+(width*y);
	*(o) = (r>>3) << 11 | (g >> 2) << 5 | (b >> 3);
}
/*-----------------------------------------------------------------*/
// Byte 0:  GGGBBBBB
// Byte 1:  RRRRRGGG
// short :  RRRR RGGG GGGB BBBB

#define SET_PIXR(addr, val) *(addr)=(*(addr)&~0xf800)|(((val)>>3)<<11)
#define SET_PIXG(addr, val) *(addr)=(*(addr)&~0x07e0)|(((val)>>2)<<5)
#define SET_PIXB(addr, val) *(addr)=(*(addr)&~0x001f)|(((val)>>3))

#define WEIGHTING(x,y) 1

void set_pixel16(int mode, int x, int y, int yy, int xxx, int width,
		 long long *mid_valuer, long long *mid_valueg, long long *mid_valueb)
{
	short *o;

	o=(short*)((char*)shminfo.shmaddr+ 2*x+ width*2*yy);
 

	if (!(y&1)) // even row
	{
		if ((x&1)) // odd column, red
		{
			*mid_valuer+=xxx;
#if 1
			SET_PIXR(o,xxx);
			SET_PIXR(o+1,xxx);
			if (!mode)
			{
				SET_PIXR(o+width,xxx);
				SET_PIXR(o+1+width,xxx);
			}
#endif
		}
		else  // green
		{
			 
			SET_PIXG(o+1,xxx);
			SET_PIXG(o,xxx);
			*mid_valueg+=xxx;
		}
	}
	else
	{
		if ((x&1)) // odd column green
		{	
			*mid_valueg+=xxx;
			SET_PIXG(o+1,xxx);
			SET_PIXG(o+0,xxx);
		}
		else //blue
		{
			*mid_valueb+=xxx;
#if 1
			SET_PIXB(o,xxx);
			SET_PIXB(o+1,xxx);
			if (!mode)
			{
				SET_PIXB(o+width,xxx);
				SET_PIXB(o+1+width,xxx);
			}
#endif
		}
	}
}



/*DECODING HACK */

#ifdef DECODING_HACK
__u8 decodebuffer[1024000];
extern void idct_init(void);
extern int decodeImage(__u8 *buffer, int len, int W, int H, int depth,
		       __u8 *framebuffer);
#endif DECODING_HACk



/*-----------------------------------------------------------------*/
int main(int argc, char** argv)
{
	int fd,dev;
	int W=352;
        int H=288;
	int mode=0; // 0:352*288, 1: 176*144
	char *device = "/dev/quick";
	int cam;
#ifdef PROFILING
	int foo = 0;
#endif
	int argp = 1;

	if (argp < argc && memcmp("/dev", argv[argp], 4) == 0)
		device = argv[argp++];

	if (argp < argc)
		mode=atoi(argv[argp++]);

#ifdef EMULATE_MODE
	fd=open("white",O_RDWR);
	W=352;
	H=288;
	goto abcd;
#endif

	// Open the driver...
	fd=open(device,O_RDWR);
	if (fd==-1)
	{
		perror("/dev/quick major 180, minor 80");
		exit(0);
	}
	dev=fd;

	// See which type of sensor we have
	cam=quick_detect_sensor(fd);
	
	if (mode>1)
	{
		fprintf(stderr, "Usage: quickcam <mode>\n"
			"mode: 0 (352*288), 1 (176*144)\n");
		exit(0);
	}

	if (!mode)
	{
		if (cam==SENSOR_HDCS) 
		{
                        W=352;
                        H=288;
                }
		else if (cam==SENSOR_VV)
		{
#ifdef DECODING_HACK
			W = 320;
			H = 240;
#else
			W = 356;
			H = 292;
#endif
		}
		else
		{
                        W=320;
                        H=240;
		}
	}
	else  // mode = 1 (small)
	{
		if (cam==SENSOR_HDCS) 
		{
                        W=176;
                        H=144;
                }
		else if (cam==SENSOR_VV)
		{
			abort();
		}
		else
		{
                        W=160;
                        H=120;
		}	
	}
		
	initializeX(W,H);
	NewPictImage(W,H);

#ifdef DECODING_HACK
	decode_init(depth);
#endif

	// Disable data stream
	quick_reg_out1(fd, STV_ISO_ENABLE, 0);

	if (cam==SENSOR_HDCS)
	{
		init_hdcs1000(fd, mode);
	}
	else if (cam==SENSOR_PB)
	{
		init_pb100(fd,mode);
	}
	else if (cam==SENSOR_VV) 
	{	
		init_855(fd);

		// set exposure time and gain, 
		// play with these value for brightness
		// exposure time in scanlines.
		quick_i2c_out_vv(fd, VV_COARSEH, 0x00);
		quick_i2c_out_vv(fd, VV_COARSEL, 0xc0);  
		quick_i2c_out_vv(fd, VV_ANALOG_GAIN, 0xfc);

#ifndef DECODING_HACK  // turn off compression
		quick_reg_out1(fd, STV_SCAN_RATE, 0x10); // larger -> slower

		quick_reg_out2(fd, STV_ISO_SIZE, 0x3ff); // ISO-Size 

		quick_reg_out1(fd, STV_Y_CTRL, 1);
#endif
	}

	// Enable data stream again
	quick_reg_out1(fd, STV_ISO_ENABLE, 1);

	fprintf(stderr,"START\n");
	po=16383;

#define SHIFT {	e=d;d=c;c=b;b=a;}

	{
		uchar a,b,c,d,e;	
		int x,y,yy;
		int n,i;	
		int frame, lxxx, block, nblock;
		long long mid_valuer=0;	
		long long mid_valueg=0;	
		long long mid_valueb=0;	
		int sum=0;
		int corruptFrame;
		a=b=c=d=e=0;
		x=0;
		y=0;
		frame = nblock= 0;
		lxxx = 0;
		corruptFrame = 0;
	
		while(1)
		{
			SHIFT;
			a=getn(fd);
		  
			if (d==0x80 && (c==2) && b==0 && a==0)
			{
#ifdef DECODING_HACK
			  if (decodeImage(decodebuffer, sum, W,H,depth,
					  (__u8*)shminfo.shmaddr))
#endif
				XPutImage(dpy,win,gc,img,0,0,0,0,W,H);
				XSync(dpy,0);
//				fprintf(stderr,"------ Found Sync %02x %02x %02x, sum %i\n", a,b,c,sum);
#ifdef PROFILING
				if (foo++ > 200)
				  return 0;
#endif
#ifdef FILE_MODE
				for(n=0;n<32;n++)
				{
					putc(0x0,stdout);
					putc(0x00,stdout);
					putc(0x0,stdout);
					putc(0x00,stdout);
				}
#endif
#if 1
				if (y && (cam==SENSOR_HDCS))
					set_exposure(fd,3*max(mid_valuer,
							    max(mid_valueg,mid_valueb))/(W*y)						     );
#endif
				mid_valuer=mid_valueg=mid_valueb=0;
				x=y=0;
				frame = nblock = 0;
				corruptFrame = 0;
				lxxx = 0;
				sum=0;
				memset(shminfo.shmaddr, 0, W*H*bypp);
				continue;
			}

// 			printf("%x %x %x %x %x   %i\n",e,d,c,b,a,n);

			if (!corruptFrame && (c==2)&&((a==0)||(a==1)||(a==2)||(a==3))&&(b==0))
			{
				n=256*a+getn(fd);
				sum+=n;
//				printf("%x %x %x %x %x   %i\n",e,d,c,b,a,n);
				if (frame > 200000)
				  continue;
#ifdef DECODING_HACK
				getbuf(fd, decodebuffer + frame, n);
				frame += n;
#else
				for(i=0;i<n;i++)
				{
					uchar xxx;
				
					xxx=(uchar)getn(fd);
#ifdef FILE_MODE
					putc(xxx,stdout);
#else
					if (mode)
						yy=y/2;
					else yy=y;

					if (yy>XH)
						yy=XH;
				
					if (depth==24)
						set_pixel24(mode,x,y,yy,xxx,W,
							    &mid_valuer,&mid_valueg,&mid_valueb);
					else
						set_pixel16(mode,x,y,yy,xxx,W,
							    &mid_valuer,&mid_valueg,&mid_valueb);
#endif
					x++;
					
					if (x==W)
					{
						x=0;
						y++;
						
					} 
					lxxx = xxx;
				}
#endif
				SHIFT;
				a=getn(fd);
				SHIFT;
				a=getn(fd);
			}
		}
	}
	
}
