/*****************************************************************************
 *
 *  stv6x.c - I2C/register access routines for the STV*-ASICs
 *
 *  Copyright (C) 2000 Georg Acher (acher@in.tum.de)
 *  Copyright (C) 2000 Mark Cave-Ayland (mca198@ecs.soton.ac.uk)
 *
 *
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/usb.h>

typedef unsigned char __u8;
typedef unsigned short __u16;

typedef struct {
        __u8 requesttype;
        __u8 request;
        __u16 value;
        __u16 index;
        __u16 length;
} devrequest __attribute__ ((packed));

typedef unsigned char uchar;

#include "quickcamex.h"
#include "stv6x.h"
#include "hdcs1000.h"
#include "pb0100.h"
#include "vv6410.h"

extern unsigned char SENSOR_ADDR;
/*-----------------------------------------------------------------*/
// I2C-Output one byte for HDCS1000
void quick_i2c_out(int fd, int reg, int val)
{
	ctrl_transfer_t ct;
	
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE;
	ct.setup.index=0;
	ct.size=0x23;

	ct.data[0x20]=SENSOR_ADDR;
	ct.data[0x21]=0; // 1 value
	ct.data[0x22]=1; // Write cmd

	ct.data[0]=reg;
	ct.data[0x10]=val;

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);	
}
/*-----------------------------------------------------------------*/
// I2C-Output 16bit for PB100
void quick_i2c_out2(int fd, int reg, int val)
{
        ctrl_transfer_t ct;
        
        ct.setup.requesttype=0x40;
        ct.setup.request=0x04;
        ct.setup.value=STV_I2C_WRITE;
        ct.setup.index=0;
        ct.size=0x23;
	
        ct.data[0x20]=SENSOR_ADDR;
        ct.data[0x21]=0; // 1 value
        ct.data[0x22]=1; // Write cmd
	
        ct.data[0]=reg;

        ct.data[0x10]=val&255;
        ct.data[0x11]=(val>>8)&255;
	
        ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
// Output one Byte with additional obscure access for VV6410
// The second access Seems to be not really needed...

void quick_i2c_out_vv(int fd, int reg, int val)
{
	ctrl_transfer_t ct;
	
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE;
	ct.setup.index=0;
	ct.size=0x23;

	ct.data[0x20]=VV6410_ADDR;
	ct.data[0x21]=0; // 1 value
	ct.data[0x22]=1; // Write cmd

	ct.data[0]=reg;
	ct.data[0x10]=val;

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);

	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=0x1704;
	ct.setup.index=0;
	ct.size=0x1;
	ct.data[0]=1;
	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
// Output n bytes to n registers
void i2c_out(int fd, int i2c_addr, int cmd, int len, char *reg, char *value)
{
	ctrl_transfer_t ct;
	int n;

	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=STV_I2C_WRITE;
	ct.setup.index=0;
	ct.size=0x23;

	ct.data[0x20]=i2c_addr;
	ct.data[0x21]=len-1;
	ct.data[0x22]=cmd;

	for(n=0;n<len;n++)
	{
		ct.data[n]=reg[n];
		ct.data[n+16]=value[n];
	}

	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
// Output one byte to STV-reg
void quick_reg_out1(int fd, int reg, int val)
{
	ctrl_transfer_t ct;
	
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=1;

	ct.data[0]=val;
	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
// Output 16bit to STV-reg
void quick_reg_out2(int fd, int reg, int val)
{
	ctrl_transfer_t ct;
	
	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=2;

	ct.data[0]=val&255;
	ct.data[1]=(val>>8)&255;
	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
// Read STV-register
int quick_reg_in(int fd, int reg)
{
	ctrl_transfer_t ct;

	ct.setup.requesttype=0xc0;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=0x1;

	ct.data[0]=0xaa;

	ioctl(fd,IOCTL_QUICKCAM_CTRLR,&ct);
	return ct.data[0];
}
/*-----------------------------------------------------------------*/
// Output n bytes to STV-registers
void quick_reg_outn(int fd, int reg, int len, char *data)
{
	ctrl_transfer_t ct;

	ct.setup.requesttype=0x40;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=len;

	memcpy(ct.data, data, len);
	ioctl(fd,IOCTL_QUICKCAM_CTRLW,&ct);
}
/*-----------------------------------------------------------------*/
