 /*****************************************************************************
 *
 *  vv6410.h - Vision/ST VV6410 register definitions
 *
 *  (see data sheet at http://www.vvl.co.uk)
 *
 *  Copyright (C) 2000 Georg Acher (acher@in.tum.de)
 *
 *****************************************************************************/

// I2C address for the VV6410

#define VV6410_ADDR 0x20

// I2C registers in the VV6410

#define VV_DEVICEH	0x00	// Chip identification (RO)
#define VV_DEVICEL	0x01
#define VV_STATUS0	0x02
#define VV_LINE_COUNTH 	0x03	// Current line counter value (RO) 
#define VV_LINE_COUNTL	0x04
#define VV_XENDH	0x05	// End x coordinate of image size (RO) 
#define VV_XENDL	0x06
#define VV_YENDH	0x07	// End y coordinate of image size (RO)
#define VV_YENDL	0x08
#define VV_DARK_AVGH	0x09	// avg. pixel value returned from dark line offset cancellation algorithm (RO)
#define VV_DARK_AVGL	0x0a
#define VV_BLACK_AVGH	0x0b	// avg. pixel value returned from black line offset cancellation algorithm (RO)
#define VV_BLACK_AVGL	0x0c
#define VV_STATUS1	0x0d	// x or y image coordinates have been clipped (RO)

#define VV_SETUP0	0x10	// Low-power/sleep modes & video timing
#define VV_SETUP1	0x11	// Various parameters
#define VV_SYNC_VALUE	0x12	// Contains pixel counter reset value used by external sync
#define VV_FG_MODES	0x14	// Frame grabbing modes
#define VV_PIN_MAPPING	0x15	// FST and QCK mapping modes
#define VV_DATA_FORMAT	0x16	// Data resolution
#define VV_OP_FORMAT	0x17	// Output coding formats
#define VV_MODE_SELECT	0x18	// Various mode select bits

#define VV_FINEH	0x20	// Fine exposure
#define VV_FINEL	0x21
#define VV_COARSEH	0x22	// Coarse exposure
#define VV_COARSEL	0x23
#define VV_ANALOG_GAIN	0x24	// Analog gain setting
#define VV_CLK_DIV	0x25	// Clock division

#define VV_DARK_OFFSETH	0x2c	// dark line offset cancellation value
#define VV_DARK_OFFSETL	0x2d
#define VV_DARK_OFFSET_SETUP 0x2e	// dark line offset cancellation enable

#define VV_LINE_LENGTHH	0x52	// Line Length (Pixel Clocks)
#define VV_LINE_LENGTHL	0x53
#define VV_XOFFSETH 	0x57	// x-co-ordinate of top left corner
#define VV_XOFFSETL	0x58
#define VV_YOFFSETH	0x59	// y-co-ordinate of top left corner
#define VV_YOFFSETL	0x5a

#define VV_FIELD_LENGTHH 0x61	// Field length (Lines)
#define VV_FIELD_LENGTHL 0x62

#define VV_BLACK_OFFSETH 0x70	// black offset cancellation default value
#define VV_BLACK_OFFSETL 0x71
#define VV_BLACK_OFFSET_SETUP 0x72	// black offset cancellation setup

#define VV_CR0		0x75	// Analog Control Register 0
#define VV_CR1		0x76	// Analog Control Register 1
#define VV_AS0		0x77	// ADC Setup Register
#define VV_AT0		0x78	// Analog Test Register
#define VV_AT1		0x79	// Audio Amplifier Setup Register

